package org.commonjava.tensor.flat.data.store;

import java.util.HashSet;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;

import org.apache.maven.graph.effective.EProjectGraph;
import org.apache.maven.graph.effective.EProjectNet;
import org.apache.maven.graph.effective.EProjectRelationships;
import org.apache.maven.graph.effective.ref.EProjectKey;
import org.commonjava.tensor.data.TensorDataException;
import org.commonjava.tensor.data.store.IndexStore;
import org.commonjava.tensor.data.store.TensorDataStore;
import org.commonjava.tensor.flat.config.FlatFileTensorConfiguration;
import org.commonjava.tensor.inject.TensorData;
import org.commonjava.web.json.ser.JsonSerializer;

@ApplicationScoped
public class FlatFileTensorDataStore
    implements TensorDataStore
{

    @Inject
    private FlatFileIndexStoreFactory factory;

    @Inject
    @TensorData
    private JsonSerializer serializer;

    @Inject
    private FlatFileTensorConfiguration config;

    private FlatFileIndexStoreWithMetadata<EProjectRelationships> relationships;

    private FlatFileIndexStoreWithMetadata<EProjectGraph> graphs;

    protected FlatFileTensorDataStore()
    {
    }

    public FlatFileTensorDataStore( final FlatFileIndexStoreWithMetadata<EProjectRelationships> relationships,
                                    final FlatFileIndexStoreWithMetadata<EProjectGraph> graphs )
    {
        this.relationships = relationships;
        this.graphs = graphs;
    }

    @PostConstruct
    public void initCoreIndexes()
        throws TensorDataException
    {
        if ( factory != null )
        {
            final IndexStore<EProjectKey, EProjectRelationships> relCache =
                factory.getStore( PROJECT_RELATIONSHIPS_CACHE, EProjectKey.class, EProjectRelationships.class );

            final IndexStore<EProjectKey, EProjectGraph> graphCache =
                factory.getStore( GRAPH_CACHE, EProjectKey.class, EProjectGraph.class );

            relationships =
                new FlatFileIndexStoreWithMetadata<EProjectRelationships>( PROJECT_RELATIONSHIPS_CACHE,
                                                                           EProjectRelationships.class, relCache,
                                                                           serializer, config );

            graphs =
                new FlatFileIndexStoreWithMetadata<EProjectGraph>( GRAPH_CACHE, EProjectGraph.class, graphCache,
                                                                   serializer, config );
        }
    }

    @Override
    public EProjectRelationships storeRelationshipSet( final EProjectRelationships relationships, final boolean force )
        throws TensorDataException
    {
        final EProjectKey key = relationships.getKey();
        if ( force || !this.relationships.contains( key ) )
        {
            return this.relationships.store( key, relationships );
        }

        return this.relationships.get( key );
    }

    @Override
    public EProjectNet storeGraph( final EProjectGraph graph, final boolean force )
        throws TensorDataException
    {
        final EProjectKey key = graph.getKey();
        if ( force || !graphs.contains( key ) )
        {
            return graphs.store( key, graph );
        }

        return graphs.get( key );
    }

    @Override
    public boolean containsRelationshipSet( final EProjectKey key )
    {
        return relationships.contains( key );
    }

    @Override
    public boolean containsGraph( final EProjectKey key )
    {
        return graphs.contains( key );
    }

    @Override
    public EProjectGraph getGraph( final EProjectKey key )
        throws TensorDataException
    {
        return graphs.get( key );
    }

    @Override
    public EProjectRelationships getRelationshipSet( final EProjectKey key )
        throws TensorDataException
    {
        return relationships.get( key );
    }

    @Override
    public Set<EProjectGraph> getAllGraphs()
        throws TensorDataException
    {
        final Set<EProjectGraph> result = new HashSet<EProjectGraph>( graphs.size() );
        for ( final EProjectKey key : graphs.keys() )
        {
            result.add( graphs.get( key ) );
        }

        return result;
    }

    @Override
    public Set<EProjectRelationships> getAllRelationshipSets()
        throws TensorDataException
    {
        final Set<EProjectRelationships> result = new HashSet<EProjectRelationships>( relationships.size() );
        for ( final EProjectKey key : relationships.keys() )
        {
            result.add( relationships.get( key ) );
        }

        return result;
    }

    @Override
    public Set<EProjectKey> getGraphKeys()
    {
        return graphs.keys();
    }

    @Override
    public Set<EProjectKey> getRelationshipSetKeys()
    {
        return relationships.keys();
    }

}
